@@ -12,6 +12,7 @@ from datetime import (
 )
 
 from os.path import (
+    abspath,
     dirname,
     join
 )
@@ -41,7 +42,10 @@ path.append(join(dirname(__name__), '..'))
 # INFORMATION #
 ###############
 
-with open('../pydtmc/__init__.py', 'r') as file:
+base_directory = abspath(dirname(__file__))
+init_file = join(base_directory, '../pydtmc/__init__.py')
+
+with open(init_file, 'r') as file:
     file_content = file.read()
     matches = search(r'^\s*__version__\s*=\s*[\'"]([^\'"]*)[\'"]\s*$', file_content, MULTILINE)
     current_version = matches.group(1)
@@ -91,7 +95,9 @@ nitpick_ignore = []
 intersphinx_aliases = {
     ('py:class', 'matplotlib.axes._axes.Axes'): ('py:class', 'matplotlib.axes.Axes'),
     ('py:class', 'networkx.classes.digraph.DiGraph'): ('py:class', 'networkx.DiGraph'),
-    ('py:class', 'networkx.classes.digraph.MultiDiGraph'): ('py:class', 'networkx.MultiDiGraph')
+    ('py:class', 'networkx.classes.digraph.MultiDiGraph'): ('py:class', 'networkx.MultiDiGraph'),
+    ('py:class', 'networkx.classes.multidigraph.MultiDiGraph'): ('py:class', 'networkx.MultiDiGraph'),
+    ('py:class', 'scipy.sparse.base.spmatrix'): ('py:class', 'scipy.sparse.spmatrix')
 }
 
 intersphinx_mapping = {
@@ -149,7 +155,7 @@ texinfo_documents = [(master_doc, project, project_title, author, project, 'A fr
 # FUNCTIONS #
 #############
 
-def process_intersphinx_aliases(app):
+def _process_intersphinx_aliases(app):
 
     inventories = InventoryAdapter(app.builder.env)
 
@@ -178,4 +184,4 @@ def process_intersphinx_aliases(app):
 def setup(app):
 
     app.add_config_value('intersphinx_aliases', {}, 'env')
-    app.connect('builder-inited', process_intersphinx_aliases)
+    app.connect('builder-inited', _process_intersphinx_aliases)
