@@ -3,7 +3,7 @@
 __all__ = [
     # Generic
     'ofloat', 'oint', 'ostr',
-    'tany', 'texception', 'titerable',
+    'tany', 'texception', 'titerable', 'tmapping',
     'tarray', 'oarray',
     'tmc', 'omc',
     # Lists
@@ -22,6 +22,7 @@ __all__ = [
     'tbcond', 'obcond',
     'tcache', 'ocache',
     'tdists_flex', 'odists_flex',
+    'tfitres', 'ofitres',
     'tgenres', 'ogenres',
     'tgenres_ext', 'ogenres_ext',
     'tgraph', 'ograph',
@@ -29,16 +30,15 @@ __all__ = [
     'tinterval', 'ointerval',
     'tlimit_float', 'olimit_float',
     'tlimit_int', 'olimit_int',
-    'tmc_approx', 'omc_approx',
     'tmc_dict', 'omc_dict',
     'tmc_dict_flex', 'omc_dict_flex',
-    'tmc_fit', 'omc_fit',
     'tnumeric', 'onumeric',
     'tpart', 'opart',
     'tparts', 'oparts',
     'tplot', 'oplot',
     'trand', 'orand',
     'trdl', 'ordl',
+    'tredists', 'oredists',
     'tstate', 'ostate',
     'tstates', 'ostates',
     'tstatus', 'ostatus',
@@ -76,10 +76,11 @@ from typing import (
     Dict,
     Iterable,
     List,
+    Mapping,
     Optional,
     Tuple,
     TypeVar,
-    Union
+    Union,
 )
 
 
@@ -95,6 +96,7 @@ ostr = Optional[str]
 
 tany = Any
 texception = Exception
+tmapping = Mapping
 titerable = Iterable
 
 tarray = np.ndarray
@@ -148,6 +150,9 @@ ocache = Optional[tcache]
 tdists_flex = Union[int, tlist_array]
 odists_flex = Optional[tdists_flex]
 
+tfitres = Tuple[oarray, ostr]
+ofitres = Optional[tfitres]
+
 tgenres = Tuple[oarray, ostr]
 ogenres = Optional[tgenres]
 
@@ -169,18 +174,12 @@ olimit_float = Optional[tlimit_float]
 tlimit_int = Tuple[int, bool]
 olimit_int = Optional[tlimit_int]
 
-tmc_approx = Tuple[tmc, tarray]
-omc_approx = Optional[tmc_approx]
-
 tmc_dict = Dict[Tuple[str, str], float]
 omc_dict = Optional[tmc_dict]
 
 tmc_dict_flex = Dict[Tuple[str, str], Union[float, int]]
 omc_dict_flex = Optional[tmc_dict_flex]
 
-tmc_fit = Tuple[tmc, tlist_array]
-omc_fit = Optional[tmc_fit]
-
 tnumeric = Union[titerable, tarray, spsp.spmatrix, pd.DataFrame, pd.Series] if pd is not None else Union[titerable, tarray, spsp.spmatrix]
 onumeric = Optional[tnumeric]
 
@@ -199,6 +198,9 @@ orand = Optional[trand]
 trdl = Tuple[tarray, tarray, tarray]
 ordl = Optional[trdl]
 
+tredists = Union[tarray, tlist_array]
+oredists = Optional[tredists]
+
 tstate = Union[int, str]
 ostate = Optional[tstate]
 
@@ -208,7 +210,7 @@ ostates = Optional[tstates]
 tstatus = Union[int, str, tnumeric]
 ostatus = Optional[tstatus]
 
-ttfunc = Callable[[float, float], float]
+ttfunc = Callable[[int, float, int, float], float]
 otfunc = Optional[ttfunc]
 
 ttimes_in = Union[int, tlist_int]
