@@ -63,7 +63,7 @@ from .custom_types import *
 # FUNCTIONS #
 #############
 
-def extract(data: tany) -> tlist_any:
+def _extract(data: tany) -> tlist_any:
 
     result = None
 
@@ -80,7 +80,7 @@ def extract(data: tany) -> tlist_any:
     return result
 
 
-def extract_as_numeric(data: tany) -> tarray:
+def _extract_as_numeric(data: tany) -> tarray:
 
     result = None
 
@@ -113,7 +113,9 @@ def validate_boolean(value: tany) -> bool:
 
 def validate_boundary_condition(boundary_condition: tany) -> tbcond:
 
-    if isinstance(boundary_condition, float):
+    if isinstance(boundary_condition, (float, np.floating)):
+
+        boundary_condition = float(boundary_condition)
 
         if (boundary_condition < 0.0) or (boundary_condition > 1.0):
             raise ValueError('The "@arg@" parameter, when specified as a float, must have a value between 0 and 1.')
@@ -139,22 +141,22 @@ def validate_dictionary(d: tany) -> tmc_dict:
 
     keys = d.keys()
 
-    if not all(isinstance(key, tuple) and (len(key) == 2) and isinstance(key[0], str) and isinstance(key[1], str) for key in keys):
+    if not all(isinstance(key, tuple) and len(key) == 2 and isinstance(key[0], str) and isinstance(key[1], str) for key in keys):
         raise ValueError('The "@arg@" parameter keys must be tuples containing 2 string values.')
 
     keys = [key[0] for key in keys if key[0] == key[1]]
 
-    if not all(key is not None and (len(key) > 0) for key in keys):
+    if not all(key is not None and len(key) > 0 for key in keys):
         raise TypeError('The "@arg@" parameter keys must contain only valid string values.')
 
     values = d.values()
 
-    if not all(isinstance(value, (float, int)) for value in values):
+    if not all(isinstance(value, (float, int, np.floating, np.integer)) for value in values):
         raise ValueError('The "@arg@" parameter values must be float or integer numbers.')
 
     values = [float(value) for value in values]
 
-    if not all((value >= 0.0) and (value <= 1.0) for value in values):
+    if not all(0.0 <= value <= 1.0 for value in values):
         raise ValueError('The "@arg@" parameter values can contain only numbers between 0 and 1.')
 
     dictionary = {}
@@ -167,7 +169,9 @@ def validate_dictionary(d: tany) -> tmc_dict:
 
 def validate_distribution(distribution: tany, size: int) -> tdists_flex:
 
-    if isinstance(distribution, int):
+    if isinstance(distribution, (int, np.integer)):
+
+        distribution = int(distribution)
 
         if distribution <= 0:
             raise ValueError('The "@arg@" parameter, when specified as an integer, must be greater than or equal to 1.')
@@ -178,24 +182,21 @@ def validate_distribution(distribution: tany, size: int) -> tdists_flex:
 
         distribution_len = len(distribution)
 
-        if distribution_len == 0:
-            raise ValueError('The "@arg@" parameter, when specified as a list of vectors, must be non-empty.')
+        if distribution_len <= 1:
+            raise ValueError('The "@arg@" parameter, when specified as a list of vectors, must contain at least 2 elements.')
 
-        for i, vector in distribution:
+        for index, vector in enumerate(distribution):
 
             if not isinstance(vector, np.ndarray) or not np.issubdtype(vector.dtype, np.number):
                 raise TypeError('The "@arg@" parameter must contain only numeric vectors.')
 
-            if distribution_len <= 1:
-                raise ValueError('The "@arg@" parameter, when specified as a list of vectors, must contain at least 2 elements.')
-
             vector = vector.astype(float)
-            distribution[i] = vector
+            distribution[index] = vector
 
-            if (vector.ndim != 1) or (vector.size != size):
+            if vector.ndim != 1 or vector.size != size:
                 raise ValueError('The "@arg@" parameter must contain only vectors of size {size:d}.')
 
-            if not all(np.isfinite(x) and (x >= 0.0) and (x <= 1.0) for x in np.nditer(vector)):
+            if not all(np.isfinite(x) and 0.0 <= x <= 1.0 for x in np.nditer(vector)):
                 raise ValueError('The "@arg@" parameter must contain only vectors consisting of values between 0 and 1.')
 
             if not np.isclose(np.sum(vector), 1.0):
@@ -209,9 +210,11 @@ def validate_distribution(distribution: tany, size: int) -> tdists_flex:
 
 def validate_dpi(value: tany) -> int:
 
-    if not isinstance(value, int):
+    if not isinstance(value, (int, np.integer)):
         raise TypeError('The "@arg@" parameter must be an integer value.')
 
+    value = int(value)
+
     if value not in [75, 100, 150, 200, 300]:
         raise ValueError('The "@arg@" parameter must have one of the following values: 75, 100, 150, 200, 300.')
 
@@ -231,9 +234,11 @@ def validate_enumerator(value: tany, possible_values: tlist_str) -> str:
 
 def validate_float(value: tany, lower_limit: olimit_float = None, upper_limit: olimit_float = None) -> float:
 
-    if not isinstance(value, float):
+    if not isinstance(value, (float, np.floating)):
         raise TypeError('The "@arg@" parameter must be a float value.')
 
+    value = float(value)
+
     if not np.isfinite(value) or not np.isreal(value):
         raise ValueError('The "@arg@" parameter be a real finite float value.')
 
@@ -281,7 +286,7 @@ def validate_graph(graph: tany) -> tgraphs:
 def validate_hyperparameter(hyperparameter: tany, size: int) -> tarray:
 
     try:
-        hyperparameter = extract_as_numeric(hyperparameter)
+        hyperparameter = _extract_as_numeric(hyperparameter)
     except Exception:
         raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
@@ -290,13 +295,13 @@ def validate_hyperparameter(hyperparameter: tany, size: int) -> tarray:
 
     hyperparameter = hyperparameter.astype(float)
 
-    if (hyperparameter.ndim != 2) or (hyperparameter.shape[0] != hyperparameter.shape[1]):
+    if hyperparameter.ndim != 2 or hyperparameter.shape[0] != hyperparameter.shape[1]:
         raise ValueError('The "@arg@" parameter must be a 2d square matrix.')
 
     if hyperparameter.shape[0] != size:
         raise ValueError(f'The "@arg@" parameter size must be equal to {size:d}.')
 
-    if not all(np.isfinite(x) and np.isreal(x) and np.equal(np.mod(x, 1), 0) and (x >= 1.0) for x in np.nditer(hyperparameter)):
+    if not all(np.isfinite(x) and np.isreal(x) and np.equal(np.mod(x, 1.0), 0.0) and x >= 1.0 for x in np.nditer(hyperparameter)):
         raise ValueError('The "@arg@" parameter must contain only integer values greater than or equal to 1.')
 
     return hyperparameter
@@ -304,9 +309,11 @@ def validate_hyperparameter(hyperparameter: tany, size: int) -> tarray:
 
 def validate_integer(value: tany, lower_limit: olimit_int = None, upper_limit: olimit_int = None) -> int:
 
-    if not isinstance(value, int):
+    if not isinstance(value, (int, np.integer)):
         raise TypeError('The "@arg@" parameter must be an integer value.')
 
+    value = int(value)
+
     if lower_limit is not None:
         if lower_limit[1]:
             if value <= lower_limit[0]:
@@ -337,17 +344,17 @@ def validate_interval(interval: tany) -> tinterval:
     a = interval[0]
     b = interval[1]
 
-    if not isinstance(a, (float, int)) or not isinstance(b, (float, int)):
+    if not isinstance(a, (float, int, np.floating, np.integer)) or not isinstance(b, (float, int, np.floating, np.integer)):
         raise ValueError('The "@arg@" parameter must contain only float and integer values.')
 
     a = float(a)
     b = float(b)
 
-    if not all(np.isfinite(x) and np.isreal(x) for x in [a, b]):
-        raise ValueError('The "@arg@" parameter must contain only real finite float values and integer values.')
+    if not all(np.isfinite(x) and np.isreal(x) and x >= 0.0 for x in [a, b]):
+        raise ValueError('The "@arg@" parameter must contain only finite and real values greater than or equal to 0.0.')
 
     if a >= b:
-        raise ValueError('The "@arg@" parameter must two distinct values, and the first value must be less than the second one.')
+        raise ValueError('The "@arg@" parameter must contain two distinct values, and the first value must be less than the second one.')
 
     return a, b
 
@@ -363,7 +370,7 @@ def validate_markov_chain(mc: tany) -> tmc:
 def validate_mask(mask: tany, size: int) -> tarray:
 
     try:
-        mask = extract_as_numeric(mask)
+        mask = _extract_as_numeric(mask)
     except Exception:
         raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
@@ -372,13 +379,13 @@ def validate_mask(mask: tany, size: int) -> tarray:
 
     mask = mask.astype(float)
 
-    if (mask.ndim != 2) or (mask.shape[0] != mask.shape[1]):
+    if mask.ndim != 2 or mask.shape[0] != mask.shape[1]:
         raise ValueError('The "@arg@" parameter must be a 2d square matrix.')
 
     if mask.shape[0] != size:
         raise ValueError(f'The "@arg@" parameter size must be equal to {size:d}.')
 
-    if not all(np.isnan(x) or ((x >= 0.0) and (x <= 1.0)) for x in np.nditer(mask)):
+    if not all(np.isnan(x) or 0.0 <= x <= 1.0 for x in np.nditer(mask)):
         raise ValueError('The "@arg@" parameter can contain only NaNs and values between 0 and 1.')
 
     if np.any(np.nansum(mask, axis=1, dtype=float) > 1.0):
@@ -390,7 +397,7 @@ def validate_mask(mask: tany, size: int) -> tarray:
 def validate_matrix(m: tany) -> tarray:
 
     try:
-        m = extract_as_numeric(m)
+        m = _extract_as_numeric(m)
     except Exception:
         raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
@@ -399,7 +406,7 @@ def validate_matrix(m: tany) -> tarray:
 
     m = m.astype(float)
 
-    if (m.ndim != 2) or (m.shape[0] < 2) or (m.shape[0] != m.shape[1]):
+    if m.ndim != 2 or m.shape[0] < 2 or m.shape[0] != m.shape[1]:
         raise ValueError('The "@arg@" parameter must be a 2d square matrix with size greater than or equal to 2.')
 
     if not all(np.isfinite(x) for x in np.nditer(m)):
@@ -408,7 +415,7 @@ def validate_matrix(m: tany) -> tarray:
     return m
 
 
-def validate_partitions(partitions: tany, current_states: tlist_str) -> tparts:
+def validate_partitions(partitions: tany, current_states: tlist_str) -> tlists_int:
 
     if not isinstance(partitions, list):
         raise TypeError('The "@arg@" parameter is null or wrongly typed.')
@@ -416,7 +423,7 @@ def validate_partitions(partitions: tany, current_states: tlist_str) -> tparts:
     partitions_length = len(partitions)
     current_states_length = len(current_states)
 
-    if (partitions_length < 2) or (partitions_length >= current_states_length):
+    if partitions_length < 2 or partitions_length >= current_states_length:
         raise ValueError(f'The "@arg@" parameter must contain a number of elements between 0 and the number of existing states minus one ({current_states_length - 1:d}).')
 
     partitions_flat = []
@@ -427,14 +434,16 @@ def validate_partitions(partitions: tany, current_states: tlist_str) -> tparts:
         if not isinstance(partition, titerable):
             raise TypeError('The "@arg@" parameter must contain only array_like objects.')
 
-        partition_sorted = list(partition)
+        partition_list = list(partition)
 
-        partitions_flat.extend(partition_sorted)
-        partitions_groups.append(len(partition_sorted))
+        partitions_flat.extend(partition_list)
+        partitions_groups.append(len(partition_list))
 
-    if all(isinstance(s, int) for s in partitions_flat):
+    if all(isinstance(state, (int, np.integer)) for state in partitions_flat):
 
-        if any((s < 0) or (s >= current_states_length) for s in partitions_flat):
+        partitions_flat = [int(state) for state in partitions_flat]
+
+        if any(state < 0 or state >= current_states_length for state in partitions_flat):
             raise ValueError(f'The "@arg@" parameter subelements, when specified as integers, must be values between 0 and the number of existing states minus one ({current_states_length - 1:d}).')
 
     elif all(isinstance(s, str) for s in partitions_flat):
@@ -462,7 +471,8 @@ def validate_partitions(partitions: tany, current_states: tlist_str) -> tparts:
     partitions_offset = 0
 
     for partitions_group in partitions_groups:
-        partitions.append(partitions_flat[partitions_offset:partitions_offset+partitions_group])
+        partitions_extension = partitions_offset + partitions_group
+        partitions.append(partitions_flat[partitions_offset:partitions_extension])
         partitions_offset += partitions_group
 
     return partitions
@@ -471,7 +481,7 @@ def validate_partitions(partitions: tany, current_states: tlist_str) -> tparts:
 def validate_rewards(rewards: tany, size: int) -> tarray:
 
     try:
-        rewards = extract_as_numeric(rewards)
+        rewards = _extract_as_numeric(rewards)
     except Exception:
         raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
@@ -496,11 +506,12 @@ def validate_rewards(rewards: tany, size: int) -> tarray:
 
 def validate_state(state: tany, current_states: list) -> int:
 
-    if isinstance(state, int):
+    if isinstance(state, (int, np.integer)):
 
+        state = int(state)
         limit = len(current_states) - 1
 
-        if (state < 0) or (state > limit):
+        if state < 0 or state > limit:
             raise ValueError(f'The "@arg@" parameter, when specified as an integer, must have a value between 0 and the number of existing states minus one ({limit:d}).')
 
         return state
@@ -519,11 +530,12 @@ def validate_status(status: tany, current_states: list) -> tarray:
 
     size = len(current_states)
 
-    if isinstance(status, int):
+    if isinstance(status, (int, np.integer)):
 
+        status = int(status)
         limit = size - 1
 
-        if (status < 0) or (status > limit):
+        if status < 0 or status > limit:
             raise ValueError(f'The "@arg@" parameter, when specified as an integer, must have a value between 0 and the number of existing states minus one ({limit:d}).')
 
         result = np.zeros(size, dtype=float)
@@ -544,7 +556,7 @@ def validate_status(status: tany, current_states: list) -> tarray:
         return result
 
     try:
-        status = extract_as_numeric(status)
+        status = _extract_as_numeric(status)
     except Exception:
         raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
@@ -553,7 +565,7 @@ def validate_status(status: tany, current_states: list) -> tarray:
 
     status = status.astype(float)
 
-    if (status.ndim < 1) or ((status.ndim == 2) and (status.shape[0] != 1) and (status.shape[1] != 1)) or (status.ndim > 2):
+    if status.ndim < 1 or (status.ndim == 2 and status.shape[0] != 1 and status.shape[1] != 1) or status.ndim > 2:
         raise ValueError('The "@arg@" parameter must be a valid vector.')
 
     status = np.ravel(status)
@@ -561,7 +573,7 @@ def validate_status(status: tany, current_states: list) -> tarray:
     if status.size != size:
         raise ValueError(f'The "@arg@" parameter length must be equal to the number of states ({size:d}).')
 
-    if not all(np.isfinite(x) and (x >= 0.0) and (x <= 1.0) for x in np.nditer(status)):
+    if not all(np.isfinite(x) and 0.0 <= x <= 1.0 for x in np.nditer(status)):
         raise ValueError('The "@arg@" parameter must contain only values between 0 and 1.')
 
     if not np.isclose(np.sum(status), 1.0):
@@ -573,7 +585,7 @@ def validate_status(status: tany, current_states: list) -> tarray:
 def validate_state_names(states: tany, size: oint = None) -> tlist_str:
 
     try:
-        states = extract(states)
+        states = _extract(states)
     except Exception:
         raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
@@ -589,7 +601,7 @@ def validate_state_names(states: tany, size: oint = None) -> tlist_str:
     if states_unique < states_length:
         raise ValueError('The "@arg@" parameter must contain only unique values.')
 
-    if size is not None and (states_length != size):
+    if size is not None and states_length != size:
         raise ValueError(f'The "@arg@" parameter must contain a number of elements equal to {size:d}.')
 
     return states
@@ -599,16 +611,17 @@ def validate_states(states: tany, current_states: tlist_str, state_type: str, fl
 
     if flex:
 
-        if isinstance(states, int):
+        if isinstance(states, (int, np.integer)):
 
+            states = int(states)
             limit = len(current_states) - 1
 
-            if (states < 0) or (states > limit):
+            if states < 0 or states > limit:
                 raise ValueError(f'The "@arg@" parameter, when specified as an integer, must have a value between 0 and the number of existing states minus one ({limit:d}).')
 
             return [states]
 
-        if (state_type != 'walk') and isinstance(states, str):
+        if state_type != 'walk' and isinstance(states, str):
 
             if states not in current_states:
                 raise ValueError(f'The "@arg@" parameter, when specified as a string, must match the name of an existing state ({", ".join(current_states)}).')
@@ -616,15 +629,17 @@ def validate_states(states: tany, current_states: tlist_str, state_type: str, fl
             return [current_states.index(states)]
 
     try:
-        states = extract(states)
+        states = _extract(states)
     except Exception:
         raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
     current_states_length = len(current_states)
 
-    if all(isinstance(s, int) for s in states):
+    if all(isinstance(s, (int, np.integer)) for s in states):
 
-        if any((s < 0) or (s >= current_states_length) for s in states):
+        states = [int(state) for state in states]
+
+        if any(state < 0 or state >= current_states_length for state in states):
             raise ValueError(f'The "@arg@" parameter, when specified as a list of integers, must contain only values between 0 and the number of existing states minus one ({current_states_length - 1:d}).')
 
     elif all(isinstance(s, str) for s in states):
@@ -646,19 +661,19 @@ def validate_states(states: tany, current_states: tlist_str, state_type: str, fl
 
     states_length = len(states)
 
-    if (state_type != 'walk') and (len(set(states)) < states_length):
+    if state_type != 'walk' and len(set(states)) < states_length:
         raise ValueError('The "@arg@" parameter must contain only unique values.')
 
     if state_type == 'regular':
 
-        if (states_length < 1) or (states_length > current_states_length):
+        if states_length < 1 or states_length > current_states_length:
             raise ValueError(f'The "@arg@" parameter must contain a number of elements between 1 and the number of existing states ({current_states_length:d}).')
 
         states = sorted(states)
 
     elif state_type == 'subset':
 
-        if (states_length < 1) or (states_length >= current_states_length):
+        if states_length < 1 or states_length >= current_states_length:
             raise ValueError(f'The "@arg@" parameter must contain a number of elements between 1 and the number of existing states minus one ({current_states_length - 1:d}).')
 
         states = sorted(states)
@@ -686,7 +701,9 @@ def validate_string(value: tany) -> str:
 
 def validate_time_points(time_points: tany) -> ttimes_in:
 
-    if isinstance(time_points, int):
+    if isinstance(time_points, (int, np.integer)):
+
+        time_points = int(time_points)
 
         if time_points < 0:
             raise ValueError('The "@arg@" parameter, when specified as an integer, must be greater than or equal to 0.')
@@ -694,13 +711,17 @@ def validate_time_points(time_points: tany) -> ttimes_in:
         return time_points
 
     try:
-        time_points = extract(time_points)
+        time_points = _extract(time_points)
     except Exception:
         raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
-    if all(isinstance(tp, int) for tp in time_points):
-        if any(tp < 0 for tp in time_points):
+    if all(isinstance(time_point, (int, np.integer)) for time_point in time_points):
+
+        time_points = [int(time_point) for time_point in time_points]
+
+        if any(time_point < 0 for time_point in time_points):
             raise ValueError('The "@arg@" parameter, when specified as a list of integers, must contain only values greater than or equal to 0.')
+
     else:
         raise TypeError('The "@arg@" parameter must be either an integer or an array_like object of integers.')
 
@@ -724,7 +745,7 @@ def validate_transition_function(f: tany) -> ttfunc:
 
     s = signature(f)
 
-    if len(s.parameters) != 2:
+    if len(s.parameters) != 4:
         raise ValueError('The "@arg@" parameter must accept 2 input arguments.')
 
     return f
@@ -733,7 +754,7 @@ def validate_transition_function(f: tany) -> ttfunc:
 def validate_transition_matrix(p: tany) -> tarray:
 
     try:
-        p = extract_as_numeric(p)
+        p = _extract_as_numeric(p)
     except Exception:
         raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
@@ -742,7 +763,7 @@ def validate_transition_matrix(p: tany) -> tarray:
 
     p = p.astype(float)
 
-    if (p.ndim != 2) or (p.shape[0] != p.shape[1]):
+    if p.ndim != 2 or p.shape[0] != p.shape[1]:
         raise ValueError('The "@arg@" parameter must be a 2d square matrix.')
 
     size = p.shape[0]
@@ -750,7 +771,7 @@ def validate_transition_matrix(p: tany) -> tarray:
     if size < 2:
         raise ValueError('The "@arg@" parameter size must be greater than or equal to 2.')
 
-    if not all(np.isfinite(x) and (x >= 0.0) and (x <= 1.0) for x in np.nditer(p)):
+    if not all(np.isfinite(x) and 0.0 <= x <= 1.0 for x in np.nditer(p)):
         raise ValueError('The "@arg@" parameter must contain only values between 0 and 1.')
 
     if not np.allclose(np.sum(p, axis=1), np.ones(size, dtype=float)):
@@ -761,9 +782,11 @@ def validate_transition_matrix(p: tany) -> tarray:
 
 def validate_transition_matrix_size(size: tany) -> int:
 
-    if not isinstance(size, int):
+    if not isinstance(size, (int, np.integer)):
         raise TypeError('The "@arg@" parameter must be an integer value.')
 
+    size = int(size)
+
     if size < 2:
         raise ValueError('The "@arg@" parameter must be greater than or equal to 2.')
 
@@ -772,13 +795,11 @@ def validate_transition_matrix_size(size: tany) -> int:
 
 def validate_vector(vector: tany, vector_type: str, flex: bool, size: oint = None) -> tarray:
 
-    if flex and size is not None and isinstance(vector, int):
+    if flex and size is not None and isinstance(vector, (float, int, np.floating, np.integer)):
         vector = np.repeat(float(vector), size)
-    elif flex and size is not None and isinstance(vector, float):
-        vector = np.repeat(vector, size)
     else:
         try:
-            vector = extract_as_numeric(vector)
+            vector = _extract_as_numeric(vector)
         except Exception:
             raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
